<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
 * ResourceModel for Koongo Connector stock cache table
 *
 * @category Nostress
 * @package Nostress_Koongo
 *
 */
 
namespace Nostress\Koongo\Model\ResourceModel\Cache;

class Stock extends \Nostress\Koongo\Model\ResourceModel\Cache\Product
{	
	protected $_cacheName = 'Stock';
	protected $_mainTableAlias = self::NKCS;
	
	/**
	 * Initialize resource model
	 *
	 * @return void
	 */
	public function _construct()
	{
		$this->_init('nostress_koongo_cache_stock', 'product_id');
	}	
	
	protected function reloadTable()
	{		
 		$this->cleanMainTable();
 		$this->insertRecords();
        //Leave the QTY and stock status original and untouched
         //$this->updateConfigurableQty();
		//$this->updateBundleQty();
	}
	
	protected function cleanMainTable()
	{
		$this->helper->log(__("Clean nostress_koongo_cache_stock records for store #%1",$this->getStoreId()));
		$this->getConnection()->delete($this->getMainTable(),['store_id = ?' => $this->getStoreId()]);
    }	
    
    /*
	 * Insert records with columns qty
	*/
	protected function insertRecords()
	{
		$sql = $this->getInsertRecordsSql();
		$this->runQuery($sql,$this->getMainTable(), "Insert records. Filled columns: product_id, store_id, type_id, qty, stock_status.");
    }
    
    	/*
	 * Insert records with columns min_price, price, qty 
	 */
	protected function getInsertRecordsSql()
	{
        $mainTableAlias = self::CPE;        
		$mainTable = $this->getTable('catalog_product_entity');			
					
        $select = $this->getEmptySelect();
        $columns = array( "product_id" => $mainTableAlias.".entity_id",
                        "store_id" => "({$this->getStoreId()})",
                        "type_id" => $mainTableAlias.".type_id");

        $stockId = $this->getStockId();

        if($stockId == self::DEFAULT_STOCK_ID)
        {    
            $catInvStockStatusAlias = self::CISS;
            $catInvStockStatus = $this->getTable('cataloginventory_stock_status');
            
            $columns["qty"] = $this->helper->getRoundSql("{$catInvStockStatusAlias}.qty",0);
            $columns["stock_status"] = "{$catInvStockStatusAlias}.stock_status";
            		               
		    $select->from([$mainTableAlias => $mainTable],$columns);
		
		    $select->joinLeft(
				[$catInvStockStatusAlias => $catInvStockStatus],
				"{$mainTableAlias}.entity_id = {$catInvStockStatusAlias}.product_id AND {$catInvStockStatusAlias}.stock_id = ".self::DEFAULT_STOCK_ID." AND {$catInvStockStatusAlias}.website_id = {$this->getStockWebsiteId()}",
				null
            );	                        
        }
        else
        {
            $inventoryStockAlias = self::ISX."_".$stockId;
            $inventoryStock = $this->getTable('inventory_stock_'.$stockId);
            $columns["qty"] = $this->helper->getRoundSql("IFNULL({$inventoryStockAlias}.quantity,0)",0);
            $columns["stock_status"] = "IFNULL({$inventoryStockAlias}.is_salable,0)";
            $select->from([$mainTableAlias => $mainTable],$columns);
		
		    $select->joinLeft(
				[$inventoryStockAlias => $inventoryStock],
				"{$mainTableAlias}.sku = {$inventoryStockAlias}.sku",
				null
            );		                        
        }
        //echo $select->__toString(); exit();
		$sql = $this->getConnection()->insertFromSelect($select, $this->getMainTable(), array_keys($columns));
		
		return $sql;
	}
}